﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.DesktopEdition;
using ThinkGeo.MapSuite.Routing;

namespace RoutingServiceAreaDemo
{
    public partial class Form1 : Form
    {
        private RoutingEngine routingEngine;

        public Form1()
        {
            InitializeComponent();
        }

        private void btnGetServiceArea_Click(object sender, EventArgs e)
        {
            RtgRoutingSource routingSource = new RtgRoutingSource(@"C:\Program Files\ThinkGeo\Map Suite Routing Evaluation Edition 3.0(BETA)\HowDoISamples\SampleData\Austinstreets.rtg");
            FeatureSource featureSource = new ShapeFileFeatureSource(@"C:\Program Files\ThinkGeo\Map Suite Routing Evaluation Edition 3.0(BETA)\HowDoISamples\SampleData\Austinstreets.shp");
            routingEngine = new RoutingEngine(routingSource, featureSource);

            InMemoryFeatureLayer routingLayer = (InMemoryFeatureLayer)((LayerOverlay)winformsMap1.Overlays["RoutingOverlay"]).Layers["RoutingLayer"];

            AddPolygon(160);
            AddPolygon(80);
            AddPolygon(40);
            AddPolygon(20);

            winformsMap1.Refresh(winformsMap1.Overlays["RoutingOverlay"]);
        }

        private void AddPolygon(int distance)
        {
            PolygonShape polygonShape = routingEngine.GenerateServiceArea("5251", new TimeSpan(0, distance, 0), 2, SpeedUnit.Kph);

            InMemoryFeatureLayer routingLayer = (InMemoryFeatureLayer)((LayerOverlay)winformsMap1.Overlays["RoutingOverlay"]).Layers["RoutingLayer"];

            routingLayer.InternalFeatures.Remove(distance.ToString());

            Dictionary<string, string> columnValues = new Dictionary<string, string>();
            columnValues.Add("Distance", distance.ToString());
            Feature feature = new Feature(polygonShape, columnValues);
            routingLayer.InternalFeatures.Add(distance.ToString(), feature);
        }

        private GeoCollection<string> GetFeatureIds(IEnumerable<string> adjacentIds)
        {
            GeoCollection<string> featureIds = new GeoCollection<string>();
            foreach (string item in adjacentIds)
            {
                string featureId = item.Split(new string[] { "::" }, StringSplitOptions.RemoveEmptyEntries)[0];
                featureIds.Add(item, featureId);
            }
            return featureIds;
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            winformsMap1.MapUnit = GeographyUnit.DecimalDegree;
            winformsMap1.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.FromHtml("#e6e5d1"));
            winformsMap1.CurrentExtent = new RectangleShape(-97.763384, 30.299707, -97.712382, 30.259309);

            ShapeFileFeatureLayer austinstreetsLayer = new ShapeFileFeatureLayer(@"C:\Program Files\ThinkGeo\Map Suite Routing Evaluation Edition 3.0(BETA)\HowDoISamples\SampleData\Austinstreets.shp");
            austinstreetsLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.LocalRoad4;
            austinstreetsLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            LayerOverlay austinstreetsOverlay = new LayerOverlay();
            austinstreetsOverlay.Layers.Add("AustinstreetsLayer", austinstreetsLayer);
            winformsMap1.Overlays.Add("AustinstreetsOverlay", austinstreetsOverlay);

            InMemoryFeatureLayer routingLayer = new InMemoryFeatureLayer();
            routingLayer.Open();
            routingLayer.Columns.Add(new FeatureSourceColumn("Distance"));
            routingLayer.Close();
            routingLayer.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(PointStyles.City1);
            routingLayer.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(GetStyle());
            routingLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            LayerOverlay routingOverlay = new LayerOverlay();
            routingOverlay.Layers.Add("RoutingLayer", routingLayer);
            winformsMap1.Overlays.Add("RoutingOverlay", routingOverlay);

            austinstreetsLayer.Open();
            Feature startRoad = austinstreetsLayer.FeatureSource.GetFeatureById("5251", ReturningColumnsType.NoColumns);
            Dictionary<string, string> columnValues = new Dictionary<string, string>();
            columnValues.Add("Distance", "0");
            routingLayer.InternalFeatures.Add(new Feature(startRoad.GetShape().GetCenterPoint(), columnValues));
            austinstreetsLayer.Close();


            winformsMap1.Refresh();
        }

        private Style GetStyle()
        {
            ClassBreakStyle style = new ClassBreakStyle("Distance");
            ClassBreak classBreak20 = new ClassBreak(0, new AreaStyle(new GeoSolidBrush(GeoColor.FromArgb(100, GeoColor.StandardColors.Blue))));
            ClassBreak classBreak40 = new ClassBreak(21, new AreaStyle(new GeoSolidBrush(GeoColor.FromArgb(100, GeoColor.StandardColors.Red))));
            ClassBreak classBreak80 = new ClassBreak(41, new AreaStyle(new GeoSolidBrush(GeoColor.FromArgb(100, GeoColor.StandardColors.Yellow))));
            ClassBreak classBreak160 = new ClassBreak(81, new AreaStyle(new GeoSolidBrush(GeoColor.FromArgb(100, GeoColor.StandardColors.Green))));

            style.ClassBreaks.Add(classBreak20);
            style.ClassBreaks.Add(classBreak40);
            style.ClassBreaks.Add(classBreak80);
            style.ClassBreaks.Add(classBreak160);
            return style;
        }
    }
}
